// @flow

import RNFS from 'react-native-fs';
import { NativeModules } from 'react-native';

const FsUtils =  NativeModules.ZKFsCacheManager;

export default (
  function () {
    const cachedDir = new Set(); // flow does not know [...aSet]

    return {
      registerCacheDir(path:string) {
        if (!path || !path.length) {
          return;
        }
        path = this.fixedPath(path);
        const items = cachedDir.entries();
        let it;
        const toRm = new Set();

        do {
          it = items.next();
          const value = it && it.value;
          if (value && value[0]) {
            if (path.startsWith(value[0])) {
              return true;
            } else if (value[0].startsWith(path)) {
              toRm.add(value[0]);
            }
          }
        } while (it && !it.done);

        toRm.forEach(
          item => cachedDir.delete(item)
        );
        cachedDir.add(path);
      },

      isRegistered(path:string) {
        if (!path || !path.length) {
          return false;
        }
        path = this.fixedPath(path);
        const items = cachedDir.entries();
        let it;
        do {
          it = items.next();
          const value = it && it.value;
          if (value && value[0]) {
            if (path.startsWith(value[0])) {
              return true;
            }
          }
        } while (it && !it.done);
        return false;
      },

      fixedPath(path:string) {
        if (!path || !path.length) {
          return path;
        }
        path = path.replace(/\/{2,}/g, '/');
        const tail = path.substr(path.length - 1);
        if (tail !== '/') {
          return `${path}/`;
        } else {
          return path;
        }
      },

      cacheSize(): Promise<number | Object > {
        if (cachedDir.size <= 0) {
          return Promise.resolve(0);
        }
        return new Promise((resolver, rejecter) => {
          FsUtils.getTotalDirectorySize([...cachedDir], (error, size) => {
            if (error) {
              rejecter(error);
            } else {
              resolver(size);
            }
          });
        });
      },

      async clearCache() {
        if (cachedDir.size <= 0) {
          return false;
        }
        const dirs = [...cachedDir];
        const dirsExists = await Promise.all(dirs.map(d => RNFS.exists(d)));
        const toRm = dirs.filter((d, ii) => dirsExists[ii]);
        try {
          await Promise.all(toRm.map(aFile => RNFS.unlink(aFile)));
          return true;
        } catch (e) {
          console.error('failed to remove cachedDirs, error: ', toRm, e);
          return false;
        }
      }
    };
  }
)();
