
#import "ZKFsCacheManager.h"

@implementation ZKFsCacheManager

RCT_EXPORT_MODULE()


RCT_EXPORT_METHOD(getDirectorySize:(NSString*)path callback:(RCTResponseSenderBlock)callback) {
  if (callback) {
    callback(@[[NSNull null], @([self sizeOfDirectory:path])]);
  }
}

RCT_EXPORT_METHOD(getTotalDirectorySize:(NSArray<NSString*>*)paths callback:(RCTResponseSenderBlock)callback) {
  unsigned long long __block ret = 0;
  [paths enumerateObjectsUsingBlock:^(NSString * _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
    ret += [self sizeOfDirectory:obj];
  }];
  
  if (callback) {
    callback(@[[NSNull null], @(ret)]);
  }
}

- (unsigned long long)sizeOfDirectory:(NSString*)root {
  if (!root) {
    return 0;
  }
  unsigned long long ret = 0;
  NSFileManager *fm = [NSFileManager defaultManager];
  NSDirectoryEnumerator *directoryEnumerator = [fm enumeratorAtURL:[NSURL fileURLWithPath:root]
                                        includingPropertiesForKeys:@[NSURLIsRegularFileKey, NSURLFileSizeKey]
                                                           options: NSDirectoryEnumerationSkipsHiddenFiles
                                                      errorHandler:nil];
  for (NSURL *file in directoryEnumerator) {
    NSNumber *isFile;
    NSError *error = nil;
    if ([file getResourceValue:&isFile forKey:NSURLIsRegularFileKey error:&error]) {
      error = nil;
      if ([isFile boolValue]) {
        NSNumber *size = nil;
        if ([file getResourceValue:&size forKey:NSURLFileSizeKey error:&error]) {
          ret += [size longLongValue];
        }
      }
    } else {
      NSLog(@"failed to getResourceValue for file: %@, error: %@", file, error);
    }
    error = nil;
  }
  return ret;
}

@end
  
